#!/usr/bin/php
<?php

/**
 * BuddyServ IRC Services
 * 
 * Main program daemon
 * 
 * @name Buddy
 * @author Raymond Lynk <me@raymondj.ca>
 * @copyright (C) 2010, Raymond Lynk
 * @link http://buddyserv.raymondj.ca
 * @package Core
 * @version $Id: buddy.php 331 2009-11-22 04:21:18Z lynk $
 */

/*
 * 		 ______           __     __         _______                        
 * 		|   __ \.--.--.--|  |.--|  |.--.--.|     __|.-----.----.--.--.     
 * 		|   __ <|  |  |  _  ||  _  ||  |  ||__     ||  -__|   _|  |  |     
 * 		|______/|_____|_____||_____||___  ||_______||_____|__|  \___/      
 *  				    			   |_|                                 
 *                                                                         
 */

/* Load the configuration file */
require_once('config.php');

define('DEBUG',		true);
define('NODEBUG',	false);

/* Check if we are using a production
 * copy, or a development/test copy. This
 * will decide which databases to use.
 */
$found = false;

foreach ($config['devfolders'] as $null => $folder)
{
	if ($_SERVER['PWD'] == $folder)
	{
		$found = true;
		$site = DEBUG;
		break;
	}
}

foreach ($config['prodfolders'] as $null => $folder)
{
	if ($_SERVER['PWD'] == $folder)
	{
		$found = true;
		$site = NODEBUG;
		break;
	}
}

/* ensure our running path was listed 'somewhere' */
if (!$found)
{
	/* I dont know which database I should use. Make it fatal */
	die("\nPath: ".$_SERVER['PWD']." not a recognized path! Please edit config.php\n");
}

/* If we are not in debug mode, this is our database and stuff */
if (!$site)
{
	define('DB_TYPE',	$config['database']['production']['NAME']); // name that shows on service burst
	define('SQL_HOST',	$config['database']['production']['HOST']);
	define('SQL_USER',	$config['database']['production']['USER']);
	define('SQL_DB',	$config['database']['production']['DB']);
	define('SQL_PASS',	$config['database']['production']['PASS']);
	define('IRC_NAME',	$config['irc']['production']['MYNAME']); // this is the irc servername we go by.
	define('IRC_HOST',	$config['irc']['production']['HOSTNAME']); // irc host to link to
	define('IRC_PORT',	$config['irc']['production']['PORT']); // irc port
	define('IRC_PASS',	$config['irc']['production']['PASSWORD']); // link password
}

else 
{
	define('DB_TYPE',	$config['database']['development']['NAME']); // name that shows on service burst
	define('SQL_HOST',	$config['database']['development']['HOST']);
	define('SQL_USER',	$config['database']['development']['USER']);
	define('SQL_DB',	$config['database']['development']['DB']);
	define('SQL_PASS',	$config['database']['development']['PASS']);
	define('IRC_NAME',	$config['irc']['development']['MYNAME']); // this is the irc servername we go by.
	define('IRC_HOST',	$config['irc']['development']['HOSTNAME']); // irc host to link to
	define('IRC_PORT',	$config['irc']['development']['PORT']); // irc port
	define('IRC_PASS',	$config['irc']['development']['PASSWORD']); // link password
}

/* If the service box has a time offset from the database, specify
 * the number of seconds below, otherwise, comment it out */
define('TIME_OFFSET', 0);

// end config


//////////////////////////////////////////
/* Start by daemonizing the script, and setting timeout values */

if (function_exists("pcntl_fork"))
{
	echo "Attempting to fork()...\n";
	$pid = pcntl_fork();
	
	/* Check for a error */
	if ($pid == -1)
	{
		die('Unable to fork() process. Task terminating!');
	}
	
	elseif ($pid != 0)
	{
		/* this is the parent, tell the user that the script is running, and then quit */
		echo "[pid: {$pid}] BuddyServ IRC Services successfully entered background mode\n";
		exit;
	}
}

else
	echo "Unable to fork(): pcntl_fork is not available... Running in foreground.";
	
/* We are now in the child process. Start setting timeouts, and signal handling */
ini_set("max_execution_time", "0");
ini_set("max_input_time", "0");
set_time_limit(0);

declare(ticks = 1);

/* This was given errors... I havn't had time to fix it */
//sleep(10000);
//pcntl_signal(SIGHUP, "sig_handler");
//pcntl_signal(SIGKILL, "sig_handler");
//pcntl_signal(SIGSEGV, "sig_handler");
//pcntl_signal(SIGTERM, "sig_handler");

function sig_handler($signal)
{
	if ($signal == SIGHUP)
	{ }
	
	elseif ($signal == SIGKILL)
	{ echo 'Sig KILL'; }
		
	elseif ($signal == SIGSEGV)
	{ }
		
	elseif ($signal == SIGTERM)
	{ }
}
/* Core file */ 

define('FATAL',		true);
define('NOTFATAL',	false);

/* Require the non-class files we need */
require_once('includes/usermodes.php');
require_once('includes/chanmodes.php');
require_once('includes/ircraw.php');
require_once('includes/maincommands.php');

$db = new mysql(false);
$db->connect(SQL_HOST, '3306', SQL_PASS, SQL_USER, SQL_DB);
//$db->query('TRUNCATE `nickservwait`');


$me = new me(IRC_NAME, IRC_PASS, 'BuddyServ IRC Services by Lynk'); 
$irc = new irc(IRC_HOST, IRC_PORT);

$last_report_check = current_time(); 
main();

/* Begin recyclable code block */
function main()
{
	global $response_codes, $irc, $me, $monitor_chan, $monitor_active, $db, $last_report_check;
	
	if (!$irc->connect())
	{ 
		sleep(10);
		main();
	}
	
	else 
	{	
		/* identify ourselves to the uplink server */
		$irc->send(sprintf(PASS, $me->password));
		$irc->send(CAPAB);
		$irc->send(sprintf(SERVER, $me->name, 1, $me->description));
		
		$irc->send(sprintf(OPERWALL, $me->name, 'BuddyServ v'.$me->version.' Starting up...'));
		/* Initialize service clients */
		chanserv::initialize($me->chanserv['nick'], $me->chanserv['user'], $me->chanserv['host'], $me->chanserv['name'], $me->chanserv['modes']);
		nickserv::initialize($me->nickserv['nick'], $me->nickserv['user'], $me->nickserv['host'], $me->nickserv['name'], $me->nickserv['modes']);
		memoserv::initialize($me->memoserv['nick'], $me->memoserv['user'], $me->memoserv['host'], $me->memoserv['name'], $me->memoserv['modes']);
		operserv::initialize($me->operserv['nick'], $me->operserv['user'], $me->operserv['host'], $me->operserv['name'], $me->operserv['modes']);
		userbot::initialize($me->userbot['nick'], $me->userbot['user'], $me->userbot['host'], $me->userbot['name'], $me->userbot['modes']);
		monitor::initialize($me->monitor['nick'], $me->monitor['user'], $me->monitor['host'], $me->monitor['name'], $me->monitor['modes']);
			
		/* Introduce the clients on the network */
		chanserv::introduce();
		nickserv::introduce();
		memoserv::introduce();
		operserv::introduce();
		userbot::introduce();
		monitor::introduce();
		
		/* All services will remain in the coder channel from now on */
		$irc->send(sprintf(SJOIN, $me->name, time(), $me->coders_channel, '+', userbot::$client->nick.' '.monitor::$client->nick.' '.nickserv::$client->nick.' '.memoserv::$client->nick.' '.operserv::$client->nick));

		/* Maintain the loop while the connection is alive */
		while (!feof($irc->com))
		{
			$buffer = fgets($irc->com);
			
			$irc->recvtx += strlen($buffer);
			$irc->recvLines++;
			
			$buffer = rtrim($buffer);
			
			
			/* Check for required actions to perform,
			 * only after the EOB */
			if ($irc->has_burst())
			{
				idle::check_dosave();
				monitor::check_active();
				chanserv::leave_expired_channels();
				monitor::check_rrd();
			}
			
			/* Cycle the regex to find out what to do with server data being sent */
			foreach ($response_codes as $name => $preg)
			{
				if (preg_match($preg, $buffer, $matches))
				{
					if ($name == 'RPL_SJOIN')
					{
						handler::sJoin($matches[3], $matches[2], $matches[4], $matches[5]);
						break;
					}
					
					elseif ($name == 'RPL_INTRODUCE')
					{
						handler::userIntroduction($matches);
						break;
					}
					
					elseif ($name == 'RPL_SVINFO')
					{
						$irc->send(sprintf(SVINFO, $matches[1], $matches[2], $matches[3], time()));
						break;
					}
					
					elseif ($name == 'RPL_EOB')
					{
						$irc->setBurst(true);
						
						if ($me->use_rrd)
							$use_rrd = 'Graphing';
						
						else
							$use_rrd = 'No Graphing';
							
						$irc->send(sprintf(OPERWALL, $me->name, "BuddyServ v".$me->version." Sync Complete using ".DB_TYPE.'; '.$use_rrd));
						$irc->send(sprintf(EOB, $me->name));
						
						/* Process our glines */
						gline::populate();
						break;
					}
					
					elseif ($name == 'RPL_PING')
					{
						/* always pong back first.
						 * we dont want rogue commands to cause LAG notices, or worse,
						 * server pingouts. */
						$irc->send(sprintf(PONG, $matches[1]));
						
						/* ping sql to ensure connection stays alive */	
						$db->ping();
						
						userbot::cmd_report_check();
						
						/* expire glines */
						gline::run_expires();
						monitor::check_active();
                        $last_report_check = current_time();
						
						
						break;
					}
					
					elseif ($name == 'RPL_NEWNICK')
					{
						handler::nickChange($matches[1], $matches[2], $matches[3]);
						break;
					}
					
					elseif ($name == 'RPL_KICK')
					{
						handler::part($matches[3], $matches[2]);
						
						if (($matches[2] == $me->user_channel))
						{
							idle::remove($matches[3]);
							break;
						}
					}
					
					/* Handle channel PART */
					elseif ($name == 'RPL_PART')
					{
						handler::part($matches[1], $matches[2]);
						break;
					}
						
					/* There seems to be some kind of race condition when a user changes
					 * their virtual host */
					elseif ($name == 'RPL_QUIT')
					{
						if ($matches[2] == 'Changing hosts')
							break;
						
						idle::remove($matches[1]);	
						user::remuser(user::getuser($matches[1]));
					}
							
					elseif ($name == 'RPL_CVERSION')
					{
						$irc->send(sprintf(CTCP, $matches[2], $matches[1], 'VERSION BuddyServ v'.$me->version.' by Raymond Lynk (http://buddyserv.raymondj.ca)'));
						break;
					}
					
					elseif ($name == 'RPL_CPING')
					{
						$irc->send(sprintf(CTCP, $matches[2], $matches[1], 'PING '.time()));
						break;
					}
					
					elseif ($name == 'RPL_UMODE')
					{
						handler::uMode($matches[1], $matches[2]);
						break;
					}
					
					elseif ($name == 'RPL_NAMES')
					{
						$channel = $matches[2];
						
						/* ensure its the logging channel and that logging is active */
						if (($channel == $monitor_chan) and ($monitor_active))
						{	
							$members = preg_replace('/[~\@%\+]/', '', $matches[3]);
							$members = explode(' ', $members);
						
							foreach ($members as $each)
							{
								idle::add($each);
							}
						}
						
						break;
					}
			
					elseif ($name == 'RPL_CMODE')
					{
						handler::cMode($matches);
						break;
					}
					
					elseif ($name == 'RPL_TOPIC')
					{
						chanserv::handletopic($matches[1], $matches[2], $matches[3]);
						break;
					}
					
					elseif (($name == 'RPL_PRIVMSG') or ($name == 'RPL_SHORT'))
					{
						$words = explode(' ', $matches[3]);
						
						/* channel message */
						if (substr($matches[2], 0, 1) == '#')
						{
							monitor::check_active();
							
							if (($matches[2] == '#presource') and ($matches[1] == 'Pr3Bot'))
							{
								preg_match("/^\[(\S+)\] => (\S+)\s*.*/", implode(' ',$words), $parts);
								
								if (count($parts))
								{
									monitor::new_notification($parts[1], $parts[2]);
								}
							}
						
							/* Parse all channel messages starting
							 * with a . to channel services */
							if (substr($matches[3], 0, 1) == '.')
							{
								chanserv::parser_channel($matches[1], $matches[2], $words);
							}
							
							/* Parse all channel messages starting
							 * with a - to userbot */
							if (substr($matches[3], 0, 1) == '-')
							{
								userbot::command_parser($matches[1], $matches[2], $words);
							}
	
							/* Parse all channel messages starting
							 * with a ! to monitor service */
							if (substr($matches[3], 0, 1) == '!')
							{
								monitor::parser($matches[1], $matches[2], $words);
							}
							
							break;
						}
						
						elseif ($matches[2] == $me->nickserv['nick'])
						{
							nickserv::parser($matches[1], $words);
							break;
						}
						
						elseif ($matches[2] == $me->operserv['nick'])
						{
							operserv::command_parser($matches[1], $words);
							break;
						}
						
						elseif ($matches[2] == $me->chanserv['nick'])
						{
							chanserv::parser($matches[1], $words);
							break;
						}
						
						elseif ($matches[2] == $me->userbot['nick'])
						{
							userbot::command_parser($matches[1], $matches[1], $words);
							break;
						}
						
						elseif ($matches[2] == $me->memoserv['nick'])
						{
							memoserv::parser($matches[1], $words);
							break;
						}
						
						break;				
					}
				
			//		echo 'PREG: '. $name ."\n";
				}		
			}

		//	 echo '>>> '. $buffer ."\n";
		}
		
		echo 'connection lost';
	}
}

/**
 * Return current time, using offset changes
 * if necessary
 * 
 * @return int
 */
function current_time()
{
	if (TIME_OFFSET)
		return (time() + TIME_OFFSET);
		
	return time();
}

/**
 * Executes shell restart script, and exits.
 * @return void
 */
function restart()
{
	exec('sh restart.sh >/dev/null &');
	exit;
}

/**
 * Convert integer file sizes to english readable.
 * 
 * @param int $size
 * @return string
 */
function byte_convert($size)
{
			$sizetype = 0;
			
			while (1)
			{
				if (($size / 1024) > 1)
				{
					$sizetype++;
					$size = ($size / 1024);
				
				//	echo "size is now: {$size} with type {$sizetype}";
				}
				
				else 
					break;
			}

			$size = sprintf("%.2f", $size);
			
			if ($sizetype == 1)
				$byte = 'KB';
			
			elseif ($sizetype == 2)
				$byte = 'MB';
				
			elseif ($sizetype == 3)
				$byte = 'GiB';
				
			elseif ($sizetype == 4)
				$byte = 'TB';
				
				$array = array(
					'size'	=>	$size,
					'byte'	=>	$byte);
					
		return $array;
}

/**
 * Magic autoloading function to require
 * classes that are not yet loaded.
 * 
 * @param string $class
 * @return void
 */
function __autoload($class)
{
	if (!class_exists($class))
	{
		if (file_exists('includes/class/Class.'. $class .'.php'))
		{
			require_once('includes/class/Class.'. $class .'.php');
		}
		
		else 
		{
			new error(true, 'Required class not found: '.$class);
		}
	}
}

?>
