<?php
/**
 * BuddyServ IRC Services
 * 
 * Benchmark class
 * 
 * @name Bench
 * @author Raymond Lynk <me@raymondj.ca>
 * @copyright (C) 2010, Raymond Lynk
 * @link http://buddyserv.raymondj.ca
 * @package Core
 * @version $Id: Class.bench.php 318 2009-11-14 04:17:22Z lynk $
 */

/*
 * 		 ______           __     __         _______                        
 * 		|   __ \.--.--.--|  |.--|  |.--.--.|     __|.-----.----.--.--.     
 * 		|   __ <|  |  |  _  ||  _  ||  |  ||__     ||  -__|   _|  |  |     
 * 		|______/|_____|_____||_____||___  ||_______||_____|__|  \___/      
 *  				    			   |_|                                 
 *                                                                         
 */
class bench
{
	private $start;
	private $stop;
	private $name;
	private $step; // 1 = microtime, 0 = time
	private $duration;
	
	/**
	 * Accepts a name and stepping type, 
	 * 1 = microtime(), 0 = time() 
	 * 
	 * @param string $name
	 * @param int $step
	 * @return void
	 */
	public function __construct($name, $step=1)
	{
		$this->name = $name;
		
		if ($step <> 1)
			$this->step = 0;
		
		else 
			$this->step = 1;
	}
	
	/**
	 * Start the benchmark
	 * @return void
	 */
	public function start()
	{
		if ($this->step)
			$this->start = microtime();
			
		else 
			$this->start = time();
	}
	
	/**
	 * Stop the benchmark
	 * 
	 * @return void
	 */
	public function stop()
	{
		if (!$this->start)
			return new error(false, 'Call to bench->stop() without a call to start()');
			
		if ($this->step)
			$this->stop = microtime();
			
		else 
			$this->stop = time();
	}
	
	/**
	 * Returns the passed duration
	 * 
	 * @return float
	 */
	public function duration()
	{
		if (!($this->start or $this->stop))
			return new error(false, 'Call to bench->duration() without start or end');
			
		$duration = ($this->stop - $this->start);
		
		return $duration;
	}
	
	/**
	 * Reset benchmark counts.
	 * 
	 * @return void
	 */
	public function reset()
	{
		$this->start = 0;
		$this->stop = 0;
	}
}

?>