<?php

/**
 * BuddyServ IRC Services
 * 
 * Channel class object
 * 
 * @name Channel
 * @author Raymond Lynk <me@raymondj.ca>
 * @copyright (C) 2010, Raymond Lynk
 * @link http://buddyserv.raymondj.ca
 * @package Ircd
 * @version $Id: Class.channel.php 331 2009-11-22 04:21:18Z lynk $
 */

/*
 * 		 ______           __     __         _______                        
 * 		|   __ \.--.--.--|  |.--|  |.--.--.|     __|.-----.----.--.--.     
 * 		|   __ <|  |  |  _  ||  _  ||  |  ||__     ||  -__|   _|  |  |     
 * 		|______/|_____|_____||_____||___  ||_______||_____|__|  \___/      
 *  				    			   |_|                                 
 *                                                                         
 */

class channel
{
	protected $users = array();
	protected $name;
	protected $active;
	protected $mlock;
	protected $ctime;
	protected $tlock = false;
	protected $regdate;
	protected $lastused;
	protected $description;
	protected $topic;
	protected $secureops = false;
	protected $access = array();
	protected $akick = array();
	protected $chankey;
	protected $founder;
	protected $dojoin = true;
	protected $restricted = false;
	protected $modes;
	protected $key;
	protected $can_reg = false;
	protected $juped = false;
	protected $limit;
	protected $bans = array();
	protected $exempts = array();
	
	private $db;
	
	static $list = array();
	
	/**
	 * Check for an instance of the given channel
	 * or creates one if not found.
	 * 
	 * @param string $channel
	 * @return channel
	 */
	static function get($channel)
	{
		$lchan = strtolower($channel);
		
		if (array_key_exists($lchan, channel::$list))
		{
			$object = &channel::$list[$lchan];
			
			if ($object instanceof channel)
				return $object;
				
		}
		
		else 
		{
			$object = new channel($channel);
			
			channel::$list[$lchan] = $object;
			$object = &channel::$list[$lchan];
			
			return $object;
		}
	}
	
	/**
	 * Check if a given channel currently exists.
	 * 
	 * @param string $channel
	 * @return bool
	 */
	static function exists($channel)
	{
		$lchan = strtolower($channel);
		
		if (array_key_exists($lchan, channel::$list))
			return true;
			
		return false;
	}
	
	/**
	 * Remove a channel from our current list
	 * of active channels.
	 * 
	 * @param channel $object
	 * @return bool
	 */
	static function remove($object)
	{
		if ($object instanceof channel)
		{
			$lchan = strtolower($object->name);
			
			if (array_key_exists($lchan, channel::$list))
			{
				unset(channel::$list[$lchan]);
				return true;
			} 
			
		}
		
		return false;
	}
	
	/**
	 * Creates a new instance of a channel. Used to store
	 * all relevant information for the duration its needed.
	 * 
	 * @param string $channel
	 * @return channel
	 */
	public function __construct($channel)
	{
		global $db;
		
		$this->name = $channel;
		$this->db = &$db;
		
		$array = $this->sql_single($channel);
		
		if (is_array($array))
		{
			$this->id = $array['id'];
			$this->access = unserialize($array['accesslist']);
			$this->akick = unserialize($array['akick']);
			$this->chankey = $array['chankey'];
			$this->dojoin = $array['dojoin'];
			$this->lastused = $array['lastused'];
			$this->active = true;
			$this->regdate = $array['regdate'];
			$this->founder = $array['founder'];
			$this->name = $array['name'];
			$this->description = $array['description'];
			$this->topic = $array['topic'];
			$this->mlock = $array['mlock'];
			
			if ($array['secureops'] == 'on')
				$this->secureops = true;
				
			else
				$this->secureops = false;
				
			if ($array['restricted'] == 'on')
				$this->restricted = true;
				
			else
				$this->restricted = false;
				
			$this->tlock = $array['topiclock'];
			
			if (!$this->access)
				$this->access = array();
		}		
	}
	
	/**
	 * Check if the channel is juped or not.
	 * Returns false or the jupe reason
	 * 
	 * @return mixed
	 */
	public function is_juped()
	{
		$ifjupe = $this->db->query('SELECT * FROM `buddyserv_jupedchans` WHERE `channel` = \''.$this->name.'\' LIMIT 1');
		
		if ($this->db->num_rows($ifjupe))
		{
			$array = $this->db->fetch_array($ifjupe);
			
			return $array['reason'];
		}
		
		else 
			return false;
	}
	
	/**
	 * Check the database for an entry for the given channel.
	 * Returns an array on success or false on failure.
	 * 
	 * @param string $channel
	 * @return mixed
	 */
	private function sql_single($channel)
	{		
		$query = $this->db->query('SELECT * FROM `buddyserv_channels` WHERE `name` = \''.$channel.'\' LIMIT 1');
		
		if ($this->db->num_rows($query) == 0)
			return false;
			
		$array = $this->db->fetch_array($query);
		
		return $array;
	}
	
	/**
	 * Return the amount of users on this channel.
	 * 
	 * @return int
	 */
	public function usercount()
	{
		return count($this->users);
	}
	
	/**
	 * Checks if the given mask is banned
	 * from this channel.
	 * 
	 * @param string $mask
	 * @return bool
	 */
	public function isBanned($mask)
	{
		if (count($this->bans) == 0)
			return false;
			
		foreach ($this->bans as $each => $time)
		{
			/* check if ban matches this mask */
			if (preg_match('/^'.make_wild($each).'$/i', $mask))
				return true;
		}
		
		/* No matches. not banned */
		return false;
	}
	
	/**
	 * Adds a ban record to the channel
	 * 
	 * @param string $mask
	 * @return void
	 */
	public function addBan($mask)
	{
		$this->bans[$mask] = time();
	}
	
	/**
	 * Remove a ban from the channel records.
	 * 
	 * @param string $mask
	 * @return void
	 */
	public function removeBan($mask)
	{
		if (array_key_exists($mask, $this->bans))
			unset($this->bans[$mask]);
	}
	
	/**
	 * Find an return a ban matching a mask.
	 * Returns false on no matches.
	 * @param string $mask
	 * @return string
	 */
	public function findMatchingBan($mask)
	{
		foreach ($this->bans as $each => $time)
		{
			/* check if ban matches this mask */
			if (preg_match('/^'.make_wild($each).'$/i', $mask))
				return $each;
		}
		
		return false;
	}
	
	/**
	 * Remove all bans matching a mask.
	 * 
	 * @param string $mask
	 * @return void
	 */
	public function removeBansMatching($mask)
	{
		foreach ($this->bans as $each => $time)
		{
			/* check if ban matches this mask */
			if (preg_match('/^'.make_wild($each).'$/i', $mask))
				$this->removeBan($each);
		}
	}
	
	/**
	 * Removes a mode from the channel bits
	 * 
	 * @param bit $bin
	 * @return void
	 */
	public function removeMode($bin)
	{
		$this->modes &= ~$bin;
		echo $this->modes;
	}
	
	public function isMember($who)
	{
		$user = strtolower($who);
		
		if (array_key_exists($user, $this->users))
			return true;
			
		return false;
	}
	
	/**
	 * Sets the channels active modes in BIN
	 * 
	 * @param bit $modes
	 * @return void
	 */
	public function addMode($modes)
	{
		$this->modes |= $modes;	
	}
	
	public function __get($what)
	{
		return $this->{$what};
	}
	
	public function __set($key, $value)
	{
		$this->{$key} = $value;
	}
	
	/**
	 * Is a registered channel?
	 * 
	 * @return bool
	 */
	public function is_registered()
	{
		if ($this->active)
			return true;
			
		return false;
	}
	
	/**
	 * Check if a user has the given access level
	 * to the channel.
	 * 
	 * @param string $nick
	 * @param bit $which
	 * @return bool
	 */
	public function has_access($nick, $which)
	{
		$lower = strtolower($nick);
		
		if (array_key_exists($lower, $this->access))
		{
			$binary = $this->access[$lower];
			
			if ($binary >= $which)
				return true;
			/*
			if (($binary & $which) > 0)
				return true;
				*/
		}
		
		return false;
	}
	
	/**
	 * Add user to the channel access list with
	 * the given level
	 * 
	 * @param string $user
	 * @param bit $binmode
	 * @return void
	 */
	public function addaccess($user, $binmode)
	{
		global $db;
		$lower = strtolower($user);
		$this->access[$lower] = $binmode;
		
		$serialize = serialize($this->access);
		$db->query("UPDATE `buddyserv_channels` SET `accesslist` = '".$serialize."' WHERE `name` = '".$this->name."' LIMIT 1");
	}
	
	/**
	 * Remove access for a user
	 * 
	 * @param string $user
	 * @param bit $binmode
	 * @return void
	 */
	public function remaccess($user, $binmode)
	{
		$lower = strtolower($user);
		unset($this->access[$lower]);
	}
	
	/**
	 * Remove a user entry from the current channel.
	 * Returns the amount of users left.
	 * 
	 * @param stirng $user
	 * @return int
	 */
	public function deluser($user)
	{
		global $irc, $me;
		
		$luser = strtolower($user);
		
		unset($this->users[$luser]);
		
		if ($this->usercount() == 0)
		{
			/* If its in the part queue, dont leave
			 * yet. This prevents clients from script
			 * joining causing a network flood.
			 */
			if (!array_key_exists($this->name, chanserv::$partchans))
				$irc->send(sprintf(PART, $me->chanserv['nick'], $this->name, '0 users. goodbye.'));
			
			return 0;
		}
		
		return count($this->usercount());
	}
	
	/**
	 * Called when a user who is on this
	 * channel changes their nickname.
	 * 
	 * @param string $user
	 * @param string $newnick
	 * @return void
	 */
	public function rename_user($user, $newnick)
	{
		$lc = strtolower($user);
		$lc_new = strtolower($newnick);
		
		$binmode = $this->users[$lc];
		
		$this->users[$lc_new]['status'] = $binmode;
		
		unset($this->users[$lc]);
	}
	
	/**
	 * Add a user to this channel with the
	 * given channel modes.
	 * 
	 * @param string $user
	 * @param bit $binmode
	 * @return void
	 */
	public function adduser($user, $binmode)
	{
		global $db;
		
		$lcuser = strtolower($user);
		
		$this->users[$lcuser]['status'] = $binmode;
		
		/* if last used time is older than 1 minute
		 * change it.
		 */
		if ((time() - $this->lastused) > 60)
		{
			$this->lastused = time();
			
			if ($this->active)
				$db->query("UPDATE `buddyserv_channels` SET `lastused` = ".time()." WHERE `name` = '".$this->name."' LIMIT 1");
		}
	}
	
	/**
	 * Add a channel status to the given user.
	 * 
	 * @param string $user
	 * @param bit $bin
	 * @return void
	 */
	public function addUserStatus($user, $bin)
	{
		$lcuser = strtolower($user);
		$this->users[$lcuser]['status'] |= $bin;
	}
	
	/**
	 * Remove a channel user status from a user
	 * 
	 * @param string $user
	 * @param bit $bin
	 * @return void
	 */
	public function removeUserStatus($user, $bin)
	{
		$lcuser = strtolower($user);
		$this->users[$lcuser]['status'] &= ~$bin;
	}
	
	/**
	 * Check if the user has a given status.
	 * 
	 * @param string $user
	 * @param bit $bin
	 * @return void
	 */
	public function hasUserStatus($user, $bin)
	{
		$lcuser = strtolower($user);
		
		if (($this->users[$lcuser]['status'] & $bin) > 0)
			return true;
			
		return false;
	}
	
	/**
	 * Convert IRCD mode prefixes to their
	 * letter equivalence -- not sure if this is used...
	 * 
	 * @param $mask
	 * @param $modes
	 * @return unknown_type
	 */
	public function add_auto_status1($mask, $modes)
	{
		switch ($modes)
		{
			case '@':
			case '~':
				$modes = '+o';
				break;
			
			case '%':
				$modes = '+h';
				break;
				
			case '+':
				$modes = '+v';
				
			default:
				$modes = '';
		}
	}

}

?>