<?php

/**
 * BuddyServ IRC Services
 * 
 * Idle bonus handler. Provides bonuses for
 * members who are active in the IRC channel.
 * 
 * @name Idle
 * @author Raymond Lynk <me@raymondj.ca>
 * @copyright (C) 2010, Raymond Lynk
 * @link http://buddyserv.raymondj.ca
 * @package Ircd
 * @version $Id: Class.idle.php 318 2009-11-14 04:17:22Z lynk $
 */

/*
 * 		 ______           __     __         _______                        
 * 		|   __ \.--.--.--|  |.--|  |.--.--.|     __|.-----.----.--.--.     
 * 		|   __ <|  |  |  _  ||  _  ||  |  ||__     ||  -__|   _|  |  |     
 * 		|______/|_____|_____||_____||___  ||_______||_____|__|  \___/      
 *  				    			   |_|                                 
 *                                                                         
 */

define('IDLE_KARMAHOUR', 2);
define('IDLE_MULTIPLIER', 2097152);
	
class idle
{
	static $currents = array();
	static $last_save;
	
	/**
	 *  Starts the idle monitor for the given nickname.
	 *  
	 *  @param string $nick
	 *  @return bool
	 */
	static function add($nick)
	{
		$nick = strtolower($nick);
		
		/* ensure the nickname doesnt have a current record */
		if (!key_exists($nick, idle::$currents))
		{
			idle::$currents[$nick] = array(
				'object' 	=> new user($nick,false,false,false,false,false),
				'start'		=> time(),
				'points'	=> 0);
			
			return true;
			
		}
		
		else
		{ 
			/* If a record already exists, it means for some reason that we missed something
			 * to unset it. This could lead to possible memory leaks as 1 time visitors slowly
			 * fill up memory with time.
			 */
			new error(false, 'idle::add('.$nick .') - resource record already exists. [possible memory leak]'); 
			return false;
		}
	}
	
	/**
	 * Get an already active idle clients object.
	 * Returns false or an idle object.
	 * 
	 * @param string $nick
	 * @return mixed
	 */
	static function get_object($nick)
	{
		$nick = strtolower(nick);
		
		if (array_key_exists($nick, idle::$currents))
		{
			return idle::$currents[$nick]['object'];
		}
		
		return false;
	}
	
	/**
	 * Get the current idle points for a user.
	 * 
	 * @param string $nick
	 * @return int
	 */
	static function get_points($nick)
	{
		$nick = strtolower($nick);
		
		if (!idle::exists($nick))
			return false;
			
		return idle::$currents[$nick]['points'];
	}
		
	/**
	 * Get the time the user started getting
	 * idle bonus this session. Returns it or false.
	 * 
	 * @param string $nick
	 * @return mixed
	 */
	static function get_time($nick)
	{
		$nick = strtolower($nick);
		
		if (array_key_exists($nick, idle::$currents))
		{
			return idle::$currents[$nick]['start'];
		}
		
		return false;
	}
	
	/**
	 * Remove a user from the idle monitoring, and save
	 * their current settings. -- currently disabled
	 * 
	 * @param $nick
	 * @param $save
	 * @return unknown_type
	 */
	static function remove($nick,$save=true)
	{
		global $irc, $log_chan, $db;
				
		$nick = strtolower($nick);
		
		if (array_key_exists($nick, idle::$currents))
		{
			if ($save)
			{
				$idletime = ((time() - idle::$currents[$nick]['start']) / 3600);
				$addbonus = round($idletime * IDLE_KARMAHOUR);
				
				if ($addbonus)
				{
					$toadd = round($addbonus * IDLE_MULTIPLIER);
			//		$query = $db->query("UPDATE `users` SET uploaded=uploaded+".$toadd." WHERE `username` = '".$nick."' LIMIT 1");
				}
				// do individual idle bonus update
			}
			
			//$irc->send(sprintf(MSG, $log_chan, "Removing {$nick} from idle watch"));
			unset(idle::$currents[$nick]);
		}
	}
	
	/**
	 * Check if the given user is getting bonuses
	 * 
	 * @param string $nick
	 * @return bool
	 */
	static function exists($nick)
	{
		$nick = strtolower($nick);
		
		if (array_key_exists($nick, idle::$currents))
			return true;
			
		return false;
	}
	
	/**
	 * Checks for all users that should receive a idle bonus
	 * and performs the updates. $if_ready can be set to false
	 * to force an update, even if the given time since last update
	 * is less than 3 hours. --- currently broken.
	 * 
	 * @param bool $if_ready
	 * @return bool
	 */
	static function check_dosave($if_ready=true)
	{
		global $irc, $me, $db;
		
		if (!idle::$last_save)
			idle::$last_save = time();
			
		$total_count = 0;
		
		/* if this is true, we only save if its time to save, if false, we force an update */
		if ($if_ready)
		{
			if (idle::$last_save < (time() - 7200))
			{
				//do compare and database server. Set the amount of people updated
				$update_count = 0;
				$total_count = count(idle::$currents); //count(idle::$currents);
				
				idle::$last_save = time();
				
			//	$irc->send(sprintf(MSG, $me->userbot['nick'], $me->support_channel, 'Performing automatic idle database updates of '.$total_count.' verified idlers.')); 
			}
			
			/* its not time to run the updates, so just return for now */
			else
				return false;
		}
		
		else 
		{
			idle::$last_save = time();
			//do compare and database server. Set the amount of people updated
			$update_count = 0;
			$total_count = count(idle::$currents); //count(idle::$currents);
			
	//		$irc->send(sprintf(MSG, $me->userbot['nick'], $me->support_channel, 'Performing manual idle database updates of '.$total_count .' verified idlers.')); 
			// do it here
		}
		
	//	if ($total_count == 0)
	//		return $irc->send(sprintf(MSG, $me->userbot['nick'], $me->support_channel, "Idle save completed with no updates"));;
			
		/* we use i to tell how to continue the where clause */
		$i=0;
		$update_count = 0;
		$where = false;
		
		foreach (idle::$currents as $each => $array)
		{
			
			
			$idletime = ((time() - $array['start']) / 3600);
			$addbonus = sprintf("%.0f", ($idletime * IDLE_KARMAHOUR));
			
			if ($addbonus != 0)
				idle::$currents[$each]['start'] = time();
			
			if ($addbonus == IDLE_KARMAHOUR)
			{
				idle::$currents[$each]['points'] = (idle::$currents[$each]['points'] + IDLE_KARMAHOUR);
				
				if ($i > 0)
					$where .= " OR `username` = '".$each."'";
			
				else
				 	$where .= "WHERE `username` = '".$each."'";
				 	
				 $update_count++;
			}
			
			else 
			{
				if ($addbonus > 0)
				{
					$toadd = sprintf("%.0f", ($addbonus * IDLE_MULTIPLIER));
					
					idle::$currents[$each]['points'] = (idle::$currents[$each]['points'] + intval($addbonus));
				//	new debug("UPDATE `users` SET uploaded=uploaded+".$toadd." WHERE `username` = '".$each."' LIMIT 1");
			//		$query = $db->query("UPDATE `users` SET uploaded=uploaded+".$toadd." WHERE `username` = '".$each."' LIMIT 1");
			//		$query = $db->query("UPDATE `users` SET seedbonus=seedbonus+".$addbonus." WHERE `username` = '".$each."' LIMIT 1");
					$update_count++;
				}
					
			}
			$i++;
		}
		
		if ($where)
		{
			$toadd = round($addbonus * IDLE_MULTIPLIER);
		//	new debug("UPDATE `users` SET uploaded=uploaded+".$toadd." ".$where);
			//$query = $db->query("UPDATE `users` SET uploaded=uploaded+".$toadd." ".$where);
		}
	//	$irc->send(sprintf(MSG, $me->userbot['nick'], $me->support_channel, "Idle save completed with {$update_count} updates"));		
	}
}
	