<?php

/**
 * Provides basic mechanisms for the IRCD
 * socket connection.
 * 
 * Channel class object
 * 
 * @name Irc
 * @author Raymond Lynk <me@raymondj.ca>
 * @copyright (C) 2010, Raymond Lynk
 * @link http://buddyserv.raymondj.ca
 * @package Ircd
 * @version $Id: Class.irc.php 331 2009-11-22 04:21:18Z lynk $
 */

/*
 * 		 ______           __     __         _______                        
 * 		|   __ \.--.--.--|  |.--|  |.--.--.|     __|.-----.----.--.--.     
 * 		|   __ <|  |  |  _  ||  _  ||  |  ||__     ||  -__|   _|  |  |     
 * 		|______/|_____|_____||_____||___  ||_______||_____|__|  \___/      
 *  				    			   |_|                                 
 *                                                                         
 */

class irc extends sockets
{
	protected $socket;
	protected $has_burst =false;
	public $sendLines = 0;
	public $recvLines = 0;
	
	/**
	 * Irc object constructor.
	 * 
	 * @param string $host
	 * @param int $port
	 * @return void
	 */
	public function __construct($host, $port)
	{
		parent::__construct($host, $port);
	}
	
	/**
	 * Disconnects from the irc server.
	 * 
	 * @return void
	 */
	public function disconnect()
	{
		fclose($this->com);	
	}
	
	/**
	 * Send a string to the ircd server
	 * 
	 * @param string $string
	 * @return bool
	 */
	public function send($string)
	{
		$string .= "\r\n";
		
		if (!$this->connected)
			return false;
			
		$this->sendto($string);
		$this->sendLines++;
		
	//	echo '<< '. $string;
		return true;
	}
	
	/**
	 * This command checks if we have finished our sync
	 * burst with the server. Plex has a tendency of making
	 * our local uplink send us an EOB on every remote server
	 * connect. This causes services to eroneously report that
	 * primary bursts are complete.
	 *
	 * @return unknown
	 */
	public function has_burst()
	{
		if ($this->has_burst)
			return true;
			
		return false;
	}
	
	/**
	 * Sets the objects burst check.
	 * 
	 * @param bool $value
	 * @return void
	 */
	public function setBurst($value)
	{
		$this->has_burst = $value;
	}
		
	
	/**
	 * Actually connected to an irc server?
	 * 
	 * @return bool
	 */
	public function ok()
	{
		return ($this->connected) ? true : false;
	}
	
}

?>
	
