<?php


/**
 * BuddyServ IRC Services
 * 
 * NickServ account identification services
 * 
 * @name NickServ
 * @author Raymond Lynk <me@raymondj.ca>
 * @copyright (C) 2010, Raymond Lynk
 * @link http://buddyserv.raymondj.ca
 * @package Clients
 * @version $Id: Class.nickserv.php 331 2009-11-22 04:21:18Z lynk $
 */

/*
 * 		 ______           __     __         _______                        
 * 		|   __ \.--.--.--|  |.--|  |.--.--.|     __|.-----.----.--.--.     
 * 		|   __ <|  |  |  _  ||  _  ||  |  ||__     ||  -__|   _|  |  |     
 * 		|______/|_____|_____||_____||___  ||_______||_____|__|  \___/      
 *  				    			   |_|                                 
 *                                                                         
 */

class nickserv
{
	static $client;
	
	/**
	 * Initialize this service client.
	 * 
	 * @param string $nick
	 * @param string $username
	 * @param string $host
	 * @param string $name
	 * @return void
	 */
	static function initialize($nick, $username, $host, $name, $modes)
	{
		self::$client = new serviceclient();
		self::$client->initialize($nick, $username, $host, $name, $modes);
	}
	
	/**
	 * Magic getter
	 * 
	 * @param string $what
	 * @return mixed
	 */
	function __get($what)
	{
		if (self::$client->$what)
			return self::$client->$what;
			
	}
	
	/**
	 * Introduce this client to the uplink server.
	 * 
	 * @return void
	 */
	static function introduce()
	{
		self::$client->introduce();
	}
	
	/**
	 * Called to parse any messages to NickServ
	 *
	 * @param string $nick
	 * @param string $arguments
	 * @return void
	 */
	static function parser($nick, $arguments)
	{
		if (!strcasecmp($arguments[0], 'ENABLE'))
			return nickserv::enable($nick, $arguments[1]);
		
		if (!strcasecmp($arguments[0], 'IDENTIFY'))
			return nickserv::identify($nick, $arguments[1]);
	
		if (!strcasecmp($arguments[0], 'VHOST'))
			return nickserv::vhost($nick, $arguments);
			
		if (!strcasecmp($arguments[0], 'LOGIN'))
			return nickserv::login($nick, $arguments[1], $arguments[2]);
			
		if (!strcasecmp($arguments[0], 'INFO'))
			return nickserv::info($nick, $arguments[1]);
			
		if (!strcasecmp($arguments[0], 'GHOST'))
			return nickserv::ghost($nick, $arguments[1], $arguments[2]);
			
		if (!strcasecmp($arguments[0], 'RECOVER'))
			return nickserv::recover($nick, $arguments[1], $arguments[2]);
			
		if (!strcasecmp($arguments[0], 'DISABLE'))
			return nickserv::disable($nick, $arguments);
		
		if (!strcasecmp($arguments[0], 'HELP'))
		{
			array_shift($arguments);
			
			if (count($arguments) == 0)
				return nickserv::$client->help($nick);
			
			return nickserv::$client->help($nick, $arguments);
		}
		
		return nickserv::unknowncmd($nick, $arguments[0]);
	}
	
		/**
	 * Command not known response
	 *
	 * @param string $nick
	 * @param string $command
	 * @return null
	 */
	static function unknowncmd($nick,$command)
	{	
		return self::$client->notice($nick, "Unknown command: \2{$command}\2; Need help? /msg ".nickserv::$client->nick." HELP");
	}
	
	/**
	 * Enables a username that is currently disabled.
	 * 
	 * @param string $nick
	 * @param string $target
	 * @return void
	 */
	static function enable($nick,$target)
	{
		global $irc, $me;
		$user = user::getuser($nick);
		
		if (($user->modes & UMODE_REGISTERED) == 0 or ($user->level < SITE_MODERATOR))
			return nickserv::denied($nick);
		
		if (!$target)
			return self::$client->notice($nick, "SYNTAX: Enable <username>");
			
		$offender = user::sql_single($target);
	
		if (!$offender)
			return self::$client->notice($nick, "The user {$target} is not currently registered.");
			
		if ($offender['class'] >= $user->level)
		{
			self::$client->notice($nick, "Oh noes! You may not enable those of equal or higher level!");
			$irc->send(sprintf(OPERWALL, self::$client->nick, $user->nick." denied trying to re-enable account:\00312".$offender['username']."\003"));
			return;
		}
			
		if ($offender['enabled'] == 'yes')
			return self::$client->notice($nick, "The user {$target} is not disabled.");
			
		user::enable($offender['username'], 'Enabled by '.$nick);
		
		$irctarget = user::getuser($target);
		
		/* if this person is online and identified, say bye */
		if (($irctarget instanceof user))
		{
			$irctarget->forbid = false;
			self::$client->notice($irctarget->nick, "This account has been re-activated. Please ensure to read the site rules and FAQ.");
		}
		
		self::$client->notice($nick, "The user {$target} has been enabled.");
		return $irc->send(sprintf(OPERWALL, self::$client->nick, $nick." enabled user '{$target}'"));
	}
	
	/**
	 * Disable a currently active username.
	 * 
	 * @param string $nick
	 * @param array $arguments
	 * @return void
	 */
	static function disable($nick,$arguments)
	{
		global $irc, $me;
		$user = user::getuser($nick);
		
		if (($user->modes & UMODE_REGISTERED) == 0 or ($user->level < SITE_MODERATOR))
			return self::denied($nick);
			
		array_shift($arguments);
		$target = array_shift($arguments);
		$reason = implode(' ', $arguments);
		
		if (!$target)
			return self::$client->notice($nick, "SYNTAX: Disable <username> <reason>");
			
		$offender = user::sql_single($target);
	
		if (!$offender)
			return self::$client->notice($nick, "The user {$target} is not currently registered.");
			
		if ($offender['class'] >= $user->level)
		{
			self::$client->notice($nick, "Oh noes! You may not disable those of equal or higher level!");
			$irc->send(sprintf(OPERWALL, self::$client->nick, $user->nick." denied trying to disable account:\00312".$offender['username']."\003. Reason: ".$reason));
			return;
		}
			
		if ($offender['enabled'] == 'no')
			return self::$client->notice($nick, "The user {$target} is already disabled.");
			
		user::disable($nick, $offender['username'], $reason);
		
		$irctarget = user::getuser($target);
		
		/* if this person is online and identified, say bye */
		if (($irctarget instanceof user) && ($irctarget->identified()))
		{
			self::$client->notice($irctarget->nick, "This account has been disabled. It may be possible to reconnect to IRC, but your services usage is revoked.");
			$irc->send(sprintf(KILL, 
			$me->nickserv['nick'], 
			$target, 
			$me->name, 
			$me->nickserv['host'], 
			$me->nickserv['username'],
			$me->nickserv['nick'],
			"Disabled: ".$reason));
			
			user::remuser($irctarget);
		}
		
		self::$client->notice($nick, "The user {$target} has been disabled.");
		return $irc->send(sprintf(OPERWALL, self::$client->nick, $nick." disables user '{$target}' Reason: {$reason}"));
	}
	
	/**
	 * Sets a virtual host for a user
	 * 
	 * @param string $nick
	 * @param array $arguments
	 * @return void
	 */
	static function vhost($nick, $arguments)
	{
		global $irc, $me, $db;
		$user = user::getuser($nick);
		array_shift($arguments);
		$type = array_shift($arguments);
		$target = array_shift($arguments);
		$host = false;
		
		if (count($arguments) > 0)
			$host = array_shift($arguments);
		
		/* user has insufficant access, or is not identified. */
		if ((!$user->identified()) or ($user->level < SITE_MODERATOR))
		{
			return self::denied($nick);
		}
		
		if (strcasecmp($type, 'LIST') == 0)
		{
			$query = $db->query("SELECT vhost.*,user.username FROM `buddyserv_vhosts` vhost JOIN `users` user ON user.id=vhost.userid");
			
			if ($db->num_rows($query) == 0)
				return self::$client->notice($nick, "There are no virtual hosts in the database.");

			$i = 1;
			self::$client->notice($nick, "List of stored virtual hosts:");
			
			while ($array = $db->fetch_array($query))
			{
				self::$client->notice($nick, $i.') '.$array['username'].' ['.$array['vhost'].']');
			}
			
			return self::$client->notice($nick, "End of virtual hosts");
		}
		
		if (strcasecmp($type, 'ADD') == 0)
		{
			if (!$host)
				return self::$client->notice($nick, "SYNTAX: VHOST <add|list|del> <username> <vhost>");
			
			/* Dont let people use my special vhost */
				if (strcasecmp($host, 'buddyserv.author') == 0)
					return self::$client->notice($nick, "You may not give someone the virtual host [buddyserv.author]");
					
			/* wtf ? dont let people break shit  by using funky shit in vhosts */
			if ((!preg_match('/^[A-Za-z0-9\.-]+$/', $host)) and ($host))
			{
				self::$client->notice($nick, 'Are you trying to break me? Don\'t use weird characters.');
				$irc->send(sprintf(OPERWALL, self::$client->nick, "{$nick} using weird characters setting vhost for '{$target}' [{$host}]"));
				return;
			}
			
			/* Get an array of user information */
			$array = user::sql_single($target);
			$vhost = new vhost();
			
			/* not an array, not registered person */
			if (!$array)
				return self::$client->notice($nick, 'Virtual Hosts can only be assigned to registered users.');
					
			$vhost->findByUser($target);
	
			$vhost->owner = $array['id'];
			$vhost->setby = $nick;	
			$vhost->host = $host;
			
			/* Check for successfull save */
			if ($vhost->save() == true)
			{
				self::$client->notice($nick, 'The virtual host for '.$target.' was set to '.$host);
				$irc->send(sprintf(OPERWALL, self::$client->nick, $nick.' changes virtual host for \''.$target.'\' to '.$host));
			
			/* check if target is online and identified. set hostname */
			
				$is_online = user::getuser($target);
				
				if (($is_online) and ($is_online->identified()))
				{
					$irc->send(sprintf(RAW, ':'.$me->name.' ENCAP * SVSMODE '.$nick.' '.$object->ts .' +x'));
					$irc->send(sprintf(RAW, ':'.$me->name.' ENCAP * CHGHOST '.$target.' :'.$host));
					
					$is_online->vhost = $host;
					return self::$client->notice($target, 'Your virtual host is now: '.$host.' [approved by '.$nick.']');
				}
			}
			
			/* failed save. */
			else
			{
				return self::$client->notice($nick, "An internal error occured trying to save that virtual host.");
			}
			
		}
		
		if (strcasecmp($type, 'DEL') == 0)
		{
			$array = user::sql_single($target);
			$vhost = new vhost();
			
			$vhost->owner = $array['id'];
			
			if ($vhost->deleteByUser() == true)
			{	
				return self::$client->notice($nick, 'The virtual host for '.$target.' was reset.');
			}
			
			else
				return self::$client->notice($nick, 'No virtual host for '.$target);
		}
		
		/* unknown type */
		return self::$client->notice($nick, "Unknown VHOST command: '".$type."'");
	}
	
	/**
	 * Info command uses to show user information
	 * 
	 * @param string $nick
	 * @param string $target
	 * @return void
	 */
	static function info($nick,$target)
	{
		global $irc, $me;
		
		$client = user::getuser($nick);
		
		if (!$target)
			return self::$client->notice($nick, "SYNTAX: INFO <username>");
			
		$who = user::sql_single($target);
		
		if ((($client->modes & UMODE_REGISTERED) > 0) and ($client->level > SITE_UPLOADER))
			$specials = true;
		
		else 
			$specials = false;
		
		if (!$who)
			return self::$client->notice($nick, "User {$target} is not a registered member.");
			
			
			if ((!$specials) and $who['enabled'] == 'no')
				return self::$client->notice($nick, "User \2{$target}\2 is disabled and may not be used.");
				
			$lines = array();
			$check_online = user::getuser($target);
	
			if ($who['enabled'] == 'no')
					$lines[] = "*** \2ACCOUNT IS DISABLED\2 ***";
				
			if (!$check_online)
			{
					$lines[] = $target.' is not currently online.';
					
			}
			
			else
			{ 
				$lines[] = $target.' is '.$check_online->name;
	
				if (!$check_online->identified())
					$lines[] = '    Not Identified to NickServ';
							
				if (!$specials)
					$lines[] = '     On IRC from: '. $check_online->username.'@'.$check_online->vhost;
				
				else
					$lines[] = '     On IRC from: '.$check_online->username.'@'.$check_online->host;
			}
			
				if ($who['donor'] == 'yes')
					$lines[] = "   \0037This user is a donor\003";
			
			$level = $who['class'];
			
			if ($level)
					$lines[] = '   User Site Level: '.user::$levels[$level];
			
					$lines[] = '   Time Registered: '.strftime("%c", $who['added']);
			
			if ($specials)
			{
				$level  = $who['class'];
					$lines[] = '      Last Site IP: '.($who['ip'] ? $who['ip'] : 'unknown');
					$lines[] = '    E-mail Address: '.$who['email'];
					$lines[] = "    Profile Address: \00312".$me->url."/userdetails.php?id=".$who['id']."\003";
					
	 
				if ($who['warned'] == 'yes')
					$lines[] = "      \0034This user has an active warning\003";
					
				if ($who['modcomment'])
				{
					$msg =  explode("\n", str_replace("\r", '', $who['modcomment']));
					
					if (count($msg))
					$lines[] = "  Last Mod Comment: ".array_shift($msg);
				}
			}
			
			foreach ($lines as $message)
			{
				self::$client->notice($nick, $message);	
			}	
			
			self::$client->notice($nick, "End of INFO");
			/* use 4 spaces for positioning */
	}
	
	/**
	 * Function to send 'access denied' string
	 *
	 * @param string $nick
	 */
	static function denied($nick)
	{	
		self::$client->notice($nick, 'Access denied.');
	}
	
	/**
	 * Recovers the target nickname from a user
	 * who should not be using it.
	 * 
	 * @param string $nick
	 * @param string $target
	 * @param string $password
	 * @return void
	 */
	static function recover($nick, $target, $password)
	{
		global $irc, $me;
		
		if (!$target or !$password)
			return self::$client->notice($nick, "Syntax: RECOVER <nick> <password>");
			
		$user = user::getuser($target);
		$first = user::getuser($nick);
		
		if (!$user)
			return self::$client->notice($nick, "Nobody with nickname '".$target."' on the network");
			
		if ($nick == $target)
			return self::$client->notice($nick, "You cannot recover yourself eh.");
			
		if ($user->password != (md5($user->secret . $password . $user->secret)))	
			return self::denied($nick);
			
		/* passed all security */
		
		$time = time();
		$guest = 'Guest' . substr($time,(strlen($time) - 4)) .rand(0,100); 
		
	//	$irc->send(sprintf(RESV, $me->nickserv['nick'], $target, 'Registered: /msg '.$me->nickserv['nick'].' RELEASE '.$target.' <password> to reclaim'));
		$irc->send(sprintf(SVSNICK, $target, $user->ts, $guest, time()));
		self::$client->notice($nick, "Your imposter has been removed.");	

	}
	
	/**
 * Ghost command to remove dead client connection
 *
 * @param string $nick
 * @param string $target
 * @param string $password
 * @return void
 */
	static function ghost($nick, $target, $password)
	{
		global $irc, $me;
		
		if ((!$target) or (!$password))
			return self::$client->notice($nick, "SYNTAX: Ghost <username> <password>");
			
		$user = user::getuser($target);
		
		if (!$user)
			return self::$client->notice($nick, "Nobody with nickname '".$target."' on the network");
			
		if ($nick == $target)
			return self::$client->notice($nick, "You cannot ghost yourself eh.");
			
		if ($user->password != (md5($user->secret . $password . $user->secret)))	
			return nickserv::denied($nick);
			
		/* passed all security */
		//:lynk KILL admin :smurfs.eu.torrentmax.org!lynk.coder.torrentmax.org!~lynk!lynk (adsasd)
		
		$irc->send(sprintf(KILL, 
			$me->nickserv['nick'], 
			$target, 
			$me->name, 
			$me->nickserv['host'], 
			$me->nickserv['username'],
			$me->nickserv['nick'],
			"Ghost command from {$nick}"));
			
		self::$client->notice($nick, "The ghost using your nickname has been removed.");
		
		user::remuser($user);
	
	}
	
	/**
	 * Allows for the remote IDENTIFY of a nickname
	 * for when someone has 2 active clients.
	 * 
	 * @param string $nick
	 * @param string $target
	 * @param string $password
	 * @return void
	 */
	static function login($nick, $target, $password)
	{
		global $irc, $me;
		
		if (!$password)
			return self::$client->notice($nick, "Syntax: LOGIN <sitenick> <password>");
			
		$user = user::getuser($nick);
		$array = user::sql_single($target);
		$is_online = user::getuser($target);
	
		if (!is_array($array))
			return self::$client->notice($nick, $target." is not a registered nickname.");
			
		if ($user->id != 0)
			return self::$client->notice($nick, "The nickname you are using is registered. LOGIN cannot be performed.");
			
		if (($user->modes & UMODE_REGISTERED) > 0)
		{
			self::$client->notice($nick, "You are already identified.");
			return;
		}
		
		if ($array['enabled'] == 'no')
			return self::$client->notice($nick, "This account is disabled and may not be used.");
			
		$hash = md5($array['secret'] . $password . $array['secret']);
		
		if ($hash != $array['passhash'])
		{
			self::$client->notice($nick, "Access denied. Attempt logged.");
			$irc->send(sprintf(OPERWALL, self::$client->nick, $nick."[".$user->host."] failed 'Login as ".$target."'; Bad password."));
			
			if ($is_online instanceof user)
				$irc->send(sprintf(NOTICE, self::$client->nick, $target, $nick."[".$user->host."] failed 'Login as YOU!!'; Bad password."));
				
			return;
		}
		
		$user->level = $array['class'];
		$user->modes = ($user->modes | UMODE_REGISTERED);
		$user->remote = true;
		
		self::$client->notice($nick, "You have been granted remote access to {$target}");
		
		$irc->send(sprintf(RAW, ':'.$me->name.' ENCAP * CHGHOST '.$nick.' :'.$nick.'.remote.'.$me->userbot['host']));
		$irc->send(sprintf(RAW, ':'.$me->name.' ENCAP * SVSMODE '.$nick.' '.$user->ts .' +x'));
		
		if ($array['class'] > SITE_VIP)
			$irc->send(sprintf(OPERWALL, $me->nickserv['nick'], $nick." granted remote access to '".$target."'"));
			
		if ($is_online instanceof user)
			self::$client->notice($target, $nick." granted remote access to your nickname.");
	}
	
	/**
 * Identify command
 *
 * @param string $nick
 * @param string $password
 * @return void
 */
	static function identify($nick, $password)
	{
		global $irc, $me, $db;
		
		if (!$password)
			return self::$client->notice($nick, "Syntax: Identify <password>");
			
		$object = &user::getuser($nick);
		
		if ($object->id == 0)
			return self::notregistered($nick, 'IDENTIFY');
			
		if ($object->forbid == true)
			return self::$client->notice($nick, "This account is disabled.");
			
		if (($object->modes & UMODE_REGISTERED) > 0)
		{
			self::$client->notice($nick, "You are already identified.");
			return;
		}
		
		$hash = md5($object->secret . $password . $object->secret);
		
		if ($hash != $object->password)
		{
			self::$client->notice($nick, 'Incorrect password. Your attempt has been logged');
			$irc->send(sprintf(OPERWALL, self::$client->nick, "Failed \2IDENTIFY\2(bad password) from ".$object->nick." [".$object->host."]"));
			
			/* If the user has failed 3 identify attempts in 5minutes, kill the user from the network --lynk */
			if ($object->failed_auths > 0)
			{
				// if its been longer than 5 minutes, reset the counter.
				if ((time() - $object->last_failed_auth) > 3600)
				{
					$object->failed_auths == 1;
					$object->last_failed_auth = time();
					return;
				}
				
				/* check if this is third strike. */
				if ($object->failed_auths == 2)
				{
					$irc->send(sprintf(KILL, 
						$me->nickserv['nick'], 
						$nick, 
						$me->name, 
						$me->nickserv['host'], 
						$me->nickserv['username'],
						$me->nickserv['nick'],
						"Too many failed IDENTIFY attempts. Please use your site password."));
			
					$irc->send(sprintf(OPERWALL, self::$client->nick, "Killing \2".$object->nick."\2 for too many invalid 'identify' attempts."));
		
					user::remuser($object);
					$object->failed_auths == 0;
					return;
				}
			}
			
			$object->failed_auths++;
			$object->last_failed_auth = time();
										
			return;
		}
		
		else 
		{
			self::$client->notice($nick, "You are now recognized.");
			$object->setwait(false);
			
			$messages = new message($object->id);
			
			if ($messages->get_new(1) > 0)
				memoserv::$client->notice($nick, "You have messages waiting. To view: /msg ".memoserv::$client->nick." LIST");
			
			if (isset(user::$levels[$object->level]))
				$host = user::$levels[$object->level];
				
			else 
				$host = 'user';
			
			/* Create a vhost object to find if we need to change a users host. */
			$vhost = new vhost();

			/* Check for a vhost on record for the current users id number. */
			if ($vhost->findById($object->id, 'userid'))
			{
				$irc->send(sprintf(RAW, ':'.$me->name.' ENCAP * CHGHOST '.$nick.' :'.$vhost->host));
				$irc->send(sprintf(RAW, ':'.$me->name.' ENCAP * SVSMODE '.$nick.' '.$object->ts .' +rx'));
				self::$client->notice($nick, 'Your virtual host has been activated. ('.$vhost->host.')');
				$object->vhost = $host;
				$object->modes = ($object->modes | UMODE_REGISTERED);
			}
			
			else 
			{	
				/* If its ME, the author, I dont want a generic virtualhost
				 * because I already have my special buddyserv.author.
				 */
				if (!$object->author)
				{
					$irc->send(sprintf(RAW, ':'.$me->name.' ENCAP * CHGHOST '.$nick.' :'.$nick.'.'.$host.'.'.$me->userbot['host']));
					$irc->send(sprintf(RAW, ':'.$me->name.' ENCAP * SVSMODE '.$nick.' '.$object->ts .' +rx'));
					
					$object->vhost = $nick.'.'.$host.'.'.$me->userbot['host'];
					$object->modes = ($object->modes | UMODE_REGISTERED);
					
					self::$client->notice($nick, 'Your virtual host has been activated. ('.$nick.'.'.$host.'.'.$me->userbot['host'].')');
				}
			}
			
			if ($object->level > SITE_ADMIN)
				$irc->send(sprintf(RAW, ':'.$me->name.' ENCAP * SVSMODE '.$nick.' '.$object->ts .' +NwaAzoq'));
			/* We are going to go through the channels that the user is current on
			 * and give them their allowed status's automatically after identifying */
			
			if (is_array($object->channels))
			{
				foreach ($object->channels as $chan => $null)
				{
					$obj = channel::get($chan);
					
					if ($object->level == SITE_ADMIN)
						$irc->send(sprintf(MODE, chanserv::$client->nick, $chan, "+ao {$nick} {$nick}"));
						
					elseif ($object->level > SITE_ADMIN)
						$irc->send(sprintf(MODE, chanserv::$client->nick, $chan, "+qo {$nick} {$nick}"));
					
					elseif ($obj->founder == $object->nick)
						$irc->send(sprintf(MODE, chanserv::$client->nick, $chan, "+qo {$nick} {$nick}"));
						
					elseif ($obj->has_access($nick, chanserv::ACCESS_SOP))
						$irc->send(sprintf(MODE, chanserv::$client->nick, $chan, "+ao {$nick} {$nick}"));
						
					elseif ($obj->has_access($nick, chanserv::ACCESS_AOP))
						$irc->send(sprintf(MODE, chanserv::$client->nick, $chan, "+o {$nick}"));
						
					elseif ($obj->has_access($nick, chanserv::ACCESS_HOP))
						$irc->send(sprintf(MODE, chanserv::$client->nick, $chan, "+h {$nick}"));
						
					elseif ($obj->has_access($nick, chanserv::ACCESS_VOP))
						$irc->send(sprintf(MODE, chanserv::$client->nick, $chan, "+v {$nick}"));
				}
			}
		}
	}
	
	/**
	 * Tells user they must be registered to issue said command
	 *
	 * @param string $nick
	 * @param string $command
	 */
	static function notregistered($nick, $command)
	{	
		self::$client->notice($nick, "You must be registered for command \2".$command."\2");
	}
	
	/**
	 * Tells a user their account is disabled
	 *
	 * @param string $nick
	 */
	static function yourdisabled($nick)
	{
		self::$client->notice($nick, 'This account is disabled. Join '.$me->support_channel.' for help');
	}
	
   /**
 	* Begins the authentication notices
 	*
 	* @param string $nick
 	* @param int $level
 	* @param bool $timed
 	*/
	static function authstart($nick,$level=1,$timed=false)
	{
		global $irc;
		
		if ($level == 1)
		{
			self::$client->notice($nick, 'This nickname is registered and protected. If it is your');
			self::$client->notice($nick, "nick, please type \2/msg NickServ IDENTIFY sitepassword\2. Otherwise,");
			self::$client->notice($nick, 'please choose a different nick. (USE YOUR SITE NICKNAME)');
			return;
			
			if ($timed)
			{	
				$pid = pcntl_fork();
	
				/* Check for a error */
				if ($pid == -1)
				{
					$irc->send(sprintf(OPERWALL, self::$client->nick, 'Warning: failed to fork() in nickserv_autostart'));
				}
				
				/* check child */
				elseif ($pid == 0)
				{
					$db = new mysql(false);
					$db->connect(SQL_HOST, '3306', SQL_PASS, SQL_USER, SQL_DB);
					
					$user = user::getuser($nick);
					
					/* userlist isnt accurate!!! */
					$user->setwait(true);
					
					self::$client->notice($nick, 'You have 60 seconds to comply  before I change your nick.'); 
					sleep(20);
					
					/* check if ident */
					if (!$user->iswait())
						exit;
					
					self::$client->notice($nick, 'You have 40 seconds to comply  before I change your nick.'); 		
					sleep(20);
					
					if (!$user->iswait())
						exit;
					
					self::$client->notice($nick, 'You have 20 seconds to change your nick. Last warning.'); 	
					sleep(20);
					
					if (!$user->iswait())
						exit;
						
					self::$client->notice($nick, 'This nickname is protected. You will now receive a Guest name.'); 
					
					$time = time();
					$guest = 'Guest' . substr($time,(strlen($time) - 4)) .rand(0,100); 
					
					$user->setwait(false);
					
					$irc->send(sprintf(SVSNICK, $nick, $user->ts, $guest, time()));
					//$irc->send(sprintf(RESV, $me->nickserv['nick'], $nick, 'Protection currently active. /msg 
					exit;
				} 
			}		
		}
	}
}

?>
