<?php

/**
 * BuddyServ IRC Services
 * 
 * OperServ operator command handler client
 * 
 * @name OperServ
 * @author Raymond Lynk <me@raymondj.ca>
 * @copyright (C) 2010, Raymond Lynk
 * @link http://buddyserv.raymondj.ca
 * @package Clients
 * @version $Id: Class.operserv.php 331 2009-11-22 04:21:18Z lynk $
 */

/*
 * 		 ______           __     __         _______                        
 * 		|   __ \.--.--.--|  |.--|  |.--.--.|     __|.-----.----.--.--.     
 * 		|   __ <|  |  |  _  ||  _  ||  |  ||__     ||  -__|   _|  |  |     
 * 		|______/|_____|_____||_____||___  ||_______||_____|__|  \___/      
 *  				    			   |_|                                 
 *                                                                         
 */

class operserv
{
	static $client;
	
	/**
	 * Initialize the current service client.
	 * 
	 * @param string $nick
	 * @param string $username
	 * @param string $host
	 * @param string $name
	 * @return void
	 */
	static function initialize($nick, $username, $host, $name, $modes)
	{
		self::$client = new serviceclient();
		self::$client->initialize($nick, $username, $host, $name, $modes);
	}
	
	/**
	 * Magic getter
	 * 
	 * @param string $what
	 * @return mixed
	 */
	function __get($what)
	{
		if (self::$client->$what)
			return self::$client->$what;
			
	}
	
	/**
	 * Introduce the client to the uplink server.
	 * 
	 * @return void
	 */
	static function introduce()
	{
		self::$client->introduce();
	}
	
	/**
	 * Called to parse any messages to OperServ
	 *
	 * @param string $nick
	 * @param string $arguments
	 * @return void
	 */
	static function command_parser($nick, $arguments)
	{
		$user = user::getuser($nick);
		
		if (($user->modes & UMODE_REGISTERED) == 0)
			return self::denied($nick, $arguments[0]);
	
		if (!strcasecmp($arguments[0], 'AKILL'))
			return self::cmd_gline($nick, $arguments);
			
		if (!strcasecmp($arguments[0], 'GLOBAL'))
			return self::cmd_global($nick, $arguments);
			
		if (!strcasecmp($arguments[0], 'SAVE'))
			return self::cmd_savetodb($nick);
			
		if (!strcasecmp($arguments[0], 'LEVEL'))
			return self::cmd_level($nick, (isset($arguments[1]) ? $arguments[1] : false));
			
		if (!strcasecmp($arguments[0], 'RESTART'))
			return self::cmd_restart($nick, $arguments);
			
		if (!strcasecmp($arguments[0], 'DIE'))
			return self::cmd_die($nick, $arguments);
			
		if (!strcasecmp($arguments[0], 'STATS'))
			return self::cmd_stats($nick);
			
		if (!strcasecmp($arguments[0], 'MODE'))
			return self::cmd_mode($nick, $arguments);
			
		if (!strcasecmp($arguments[0], 'UPDATE'))
			return self::cmd_update($nick);
			
		if (!strcasecmp($arguments[0], 'HELP'))
		{
			array_shift($arguments);
			
			if (count($arguments) == 0)
				return self::$client->help($nick);
			
			return self::$client->help($nick, $arguments);
		}
		
		return operserv::unknowncmd($nick, $arguments[0]);
	}
	
	/**
	 * Calls a backtrace. Kinda useless really...
	 * 
	 * @param string $nick
	 * @param string $channel
	 * @return void
	 */
	static function backtrace($nick, $channel)
	{
		$user = user::getuser($nick);
		
		if (!$user->identified() or ($user->level < SITE_CODER))
			return operserv::denied($nick, "Backtrace");
			
		else 
			new debug("User called Backtrace", true);
	}
	
	/**
	 * Performs an SVN update of the current code.
	 * This ONLY works if current source is under
	 * SVN control.
	 * 
	 * @param string $nick
	 * @return bool
	 */
	static function cmd_update($nick)
	{
		global $me;
		
		$user = user::getuser($nick);
		
		/* Only coders can do this. */
		if ((!$user or !$user->identified() or ($user->level < SITE_CODER)))
			return self::denied($nick, "UPDATE");
		
		/* Lets try a simple check to see if we are actually under svn control */
		if (!file_exists('.svn/'))
			return self::$client->notice($nick, "I don't believe im currently under SVN Revision.");
			
		self::$client->operwall("[".$nick."] Performing SVN Update.");
		self::$client->msg($me->coders_channel, "[".$nick."] Performing SVN Update.");
		
		exec('svn update', $result);
		
		/* if only 1 line, its usually just "at version 324" line for no updates */
		if (count($result) == 1)
		{
			self::$client->operwall("SVN Update finished with no updates.");
			return self::$client->msg($me->coders_channel, "SVN Update finished with no updates.");
		}	
		
		self::$client->operwall("SVN: A restart is being forced after a successfull 'UPDATE' command");
		self::$client->msg($me->coders_channel,"@@@@ SVN Update response @@@@");
		
		/* send each line of the update to the coders channel */
		foreach ($result as $line)
		{
			self::$client->msg($me->coders_channel, $line);
		}
		
		/* automatically call the restart command for the issueing user. */
		return self::cmd_restart($nick, array('blah', 'Automatic restart issued after code update.'));
	}		
	
	/**
	 * Command not known response
	 *
	 * @param string $nick
	 * @param string $command
	 * @return null
	 */
	static function unknowncmd($nick,$command)
	{	
		return self::$client->notice($nick, "Unknown command: \2{$command}\2; Need help? /msg ".operserv::$client->nick." HELP");
	}
	
	/**
	 * Changes a channel mode
	 * 
	 * @param string $nick
	 * @param array $arguments
	 * @return void
	 */
	static function cmd_mode($nick, $arguments)
	{
		global $irc;
	
		array_shift($arguments);
		$channel = array_shift($arguments);
		$params = implode (' ', $arguments);
		$user = user::getuser($nick);
		
		if ((!$user or !$user->identified() or ($user->level < SITE_MODERATOR)))
			return self::denied($nick, implode(' ', $arguments));
			
		if (!$params)
			return self::$client->notice($nick, "Syntax: MODE #channel +ntv JOhn");
			
		if (!channel::exists($channel))
			return self::$client->notice($nick, "Channel {$channel} does not exist.");
		
		self::$client->chanMode($channel, $params);
		
		self::$client->notice($nick, "Modes on ".$channel." changed to [".$params."]");
		$irc->send(sprintf(OPERWALL, self::$client->nick, "{$nick} used MODE {$channel} [{$params}]"));
	}
				
	/**
	 * Gets current receive/sent bandwidth
	 * 
	 * @param string $nick
	 * @return void
	 */
	static function cmd_stats($nick)
	{
		global $irc, $me;
		
		$user = user::getuser($nick);
		
		if ((!$user or !$user->identified() or ($user->level < SITE_MODERATOR)))
			return self::denied($nick, 'STATS');
			
		$sendtx = convert_bytes($irc->sendtx);
		$recvrx = convert_bytes($irc->recvtx);
		$memory = convert_bytes(memory_get_usage());
		$verphp = phpversion();
		$uname = `uname -a`;
		$uptime = duration_long($me->started, time());
		
		self::$client->notice($nick, "== BuddyServ Operational Statistics");
		self::$client->notice($nick, "  Version: ".$me->version);
		self::$client->notice($nick, "   Uptime: ".$uptime);
		self::$client->notice($nick, "Bandwidth: [Recvd: ".$recvrx['size'].$recvrx['byte']." | Sent: ".$sendtx['size'].$sendtx['byte'].']');
		self::$client->notice($nick, "   Memory: ". $memory['size'].$memory['byte']);
		self::$client->notice($nick, "-");
		self::$client->notice($nick, "Server OS: ".$uname);
		self::$client->notice($nick, "Directory: ".$_SERVER['PWD']);
		self::$client->notice($nick, "Using PHP: ". $verphp);
		self::$client->notice($nick, "== END STATS");
		
	}
	
	/**
	 * Restart BuddyServ command
	 * 
	 * @param string $nick
	 * @param array $arguments
	 * @return void
	 */
	static function cmd_restart($nick, $arguments)
	{
		global $irc, $me;
		
		$user = user::getuser($nick);
		
		if ((!$user or !$user->identified() or ($user->level < SITE_SYSOP)))
			return self::denied($nick, 'RESTART');
		
		array_shift($arguments);
		
		if (count($arguments) == 0)
			return self::$client->notice($nick, "A reason must be specified for the RESTART command.");
			
		$reason = implode(' ', $arguments);
		
		/* first, run all our updates, or at least try... */
		$irc->send(sprintf(OPERWALL, self::$client->nick, "RESTART given by {$nick} [{$reason}]"));
		self::$client->notice($nick, "[restart] Performing database sync...");
		
		userbot::cmd_report_check();
		gline::run_expires();
		idle::check_dosave(false);
		
		self::$client->notice($nick, "[restart] QUIT'ing service clients and restarting.");
		
		$irc->send(sprintf(QUIT, $me->nickserv['nick'], "restart from {$nick} ({$reason})"));
		$irc->send(sprintf(QUIT, userbot::$client->nick, "restart from {$nick} ({$reason})"));
		$irc->send(sprintf(QUIT, self::$client->nick, "restart from {$nick} ({$reason})"));
		$irc->send(sprintf(QUIT, $me->chanserv['nick'], "restart from {$nick} ({$reason})"));
		$irc->send(sprintf(QUIT, $me->monitor['nick'], "restart from {$nick} ({$reason})"));
		$irc->send(sprintf(QUIT, $me->memoserv['nick'], "restart from {$nick} ({$reason})"));
		
		$irc->disconnect();
		restart();
	}
	
	/**
	 * Tells BuddyServ to exit without restart.
	 * 
	 * @param string $nick
	 * @param array $arguments
	 * @return void
	 */
	static function cmd_die($nick, $arguments)
	{
		global $irc, $me;
		
		$user = user::getuser($nick);
		
		if ((!$user or !$user->identified() or ($user->level < SITE_SYSOP)))
			return self::denied($nick, 'RESTART');
		
		array_shift($arguments);
		
		if (count($arguments) == 0)
			return self::$client->notice($nick, "A reason must be specified for the DIE command.");
			
		$reason = implode(' ', $arguments);
		
		/* first, run all our updates, or at least try... */
		$irc->send(sprintf(OPERWALL, self::$client->nick, "DIE given by {$nick} [{$reason}]"));
		self::$client->notice($nick, "[die] Performing database sync...");
		
		userbot::cmd_report_check();
		gline::run_expires();
		idle::check_dosave(false);
		
		self::$client->notice($nick, "[die] QUIT'ing service clients and exiting.");
		
		$irc->send(sprintf(QUIT, $me->nickserv['nick'], "die command from {$nick}"));
		$irc->send(sprintf(QUIT, userbot::$client->nick, "die command from {$nick}"));
		$irc->send(sprintf(QUIT, self::$client->nick, "die command from {$nick}"));
		$irc->send(sprintf(QUIT, $me->chanserv['nick'], "die command from {$nick}"));
		$irc->send(sprintf(QUIT, $me->monitor['nick'], "die command from {$nick}"));
		$irc->send(sprintf(QUIT, $me->memoserv['nick'], "die command from {$nick}"));
		
		$irc->disconnect();
		exit;

	}
	
	/**
	 * Force database saves, usually before a QUIT
	 * 
	 * @param string $nick
	 * @return void
	 */
	static function cmd_savetodb($nick)
	{	
		$user = user::getuser($nick);
		
		if ((!$user->identified()) or ($user->level < SITE_MODERATOR))
			return self::denied($nick, 'SAVE');
	
		self::$client->notice($nick, 'Running database update...');
		idle::check_dosave(false);
		self::$client->notice($nick, 'All that database stuff is done.');
	}
	
	/**
	 * Sends a message to all users on the irc network
	 * 
	 * @param string $nick
	 * @param array $message
	 * @return void
	 */
	static function cmd_global($nick,$message)
	{
		global $irc, $me;
		
		$user = user::getuser($nick);
		
		if (($user->level < SITE_ADMIN) or (!$user->identified()))
			return self::denied($nick, 'GLOBAL');
			
		array_shift($message);
		$message = implode(' ', $message);
		
		if (!$message)
			return $irc->send(sprintf(NOTICE, self::$client->nick, $nick, 'Whats the message huh?'));
		
		$irc->send(sprintf(OPERWALL, self::$client->nick, "{$nick} uses GLOBAL command [".$message."]"));
		self::$client->notice('$$*.'.$me->userbot['host'], '[Global] '.$message);
		self::$client->notice($nick, 'Global Message delivered.');
	}
	
	/**
	 * Returns your current operserv level
	 * 
	 * @param string $nick
	 * @param string $who
	 * @return void
	 */
	static function cmd_level($nick, $who=false)
	{
		$who = ($who) ? $who : $nick;
		$user = user::getuser($who);
		$level_int = $user->level;
		$level_name = user::$levels[$level_int];
		
		self::$client->notice($nick, "{$who}'s OperServ level is {$level_int} [{$level_name}]");
	}
	 
	/**
	 * Gline command handler
	 * 
	 * @param string $nick
	 * @param array $arguments
	 * @return void
	 */
	static function cmd_gline($nick,$arguments)
	{
		global $irc, $me;
		array_shift($arguments);
		$do = array_shift($arguments);
		$expire = array_shift($arguments);
		$mask = array_shift($arguments);
		$reason = implode(' ', $arguments);
		
		$nobj = user::getuser($nick);
		
		if ($nobj->level < SITE_MODERATOR)
			return self::denied($nick, 'AKill');
		
		if (!strcasecmp($do, 'LIST'))
		{
			$max = 200;
			
			if ($expire > 0)
			{
				/* make a 1000 max listing */
				if ($expire > 1000)
					$max = 1000;
					
				else 
					$max = $expire;
			}
			
			$glines_reversed = array_reverse(gline::$list);
			$i=0;
			
			self::$client->notice($nick, 'GLines - Month/Day/Year - Expires - Mask - Oper');
			
			foreach ($glines_reversed as $mask => $object)
			{
				if (!$object->mask)
					continue;
					
				$i++;
				
				/* check if max is reached */
				if ($i > $max)
					return self::$client->notice($nick, '* Listed '.$max.' most recent gline entries');
					
				$set = date("M/d/y", $object->added);
				
				if ($object->expires == 0)
					$expires = 'permanent';
				
				else	
					$expires = sprintf("%.2f", (($object->expires - current_time()) / 60));
					
				self::$client->notice($nick, "{$i}) [expires: {$expires} mins] ".$object->mask." by ".$object->oper);
				self::$client->notice($nick, "{$i}) Reason: ".$object->reason);
			}
			
			self::$client->notice($nick, 'Listed '.$max.' most recent gline entries');
			return;
		}
		
		/* Process a removal of a gline command */
		elseif (!strcasecmp($do, 'DEL'))
		{
			$mask = $expire;
			
			if (!$mask)
				return self::$client->notice($nick, 'SYNTAX: AKILL DEL <mask>');
				
				$gline = gline::get($mask);
				
				if (!$gline)
					return self::$client->notice($nick, $mask.' not found in GLines.');
				
				/* Remove the gline, put user/hostname parts in an array, and send the unkline to all servers */
				$gline = '';
				$userhost = explode('@', $mask);
				
				$irc->send(sprintf(UNKLINE, self::$client->nick, $userhost[0], $userhost[1]));
				
				if (gline::del($mask))
					return self::$client->notice($nick, $mask.' was removed from GLine list.');
				
				/* If we get here, it means our initial checks showed the gline, but it is no longer registered.
				 * This should not be possible, but better sane then severed */
				return self::$client->notice($nick, 'The GLine on '.$mask.' disappeared while being worked on.');
		}
		
		elseif (!strcasecmp($do, 'ADD'))
		{
			if ((!$expire) and ($expire != 0))
				return self::$client->notice($nick, 'SYNTAX: AKILL ADD <+3(m|d|h|w)> <mask> <reason>');
				
			preg_match('/^\+(\d+)(\w{1})$/', $expire, $matches);
			
			if (is_array($matches))
			{
				$numeric = $matches[1];
				$duration = $matches[2];
			/* not a break */
				$userhost = explode('@', $mask);
				
				$time = 0;
				
				/* process the duration amount 3d 5w 1h */
				switch ($duration)
				{
					case 's':
						$time = $numeric;
						break;
						
					case 'm':
						$time = ($numeric * 60);
						break;
						
					case 'h':
						$time = ($numeric * 3600);
						break;
						
					case 'd':
					default:
						$time = ($numeric * 86400);
						break;
						
					case 'w':
						$time = ($numeric * 604800);
						break;
				} /* end switch */
				
				if ($time == 0)
					$expiretime = 0;
				
				else 
					$expiretime = (current_time() + $time);
					
				/* Check if we already exist in the database */
				$gline = gline::get($mask);
				
				if ($gline instanceof gline)
				{
					self::$client->notice($nick, 'Akill '.$mask.' already exists');
					return;
				}
				
				$success = gline::create($expiretime, $nick, $mask, $reason);
				
				if ($success)
				{
					$gline = gline::get($mask);
					$total_ban = 0;
					/* cycle the user list, find all matching, and send byebye */
					
					$templist = user::$userlist;
					
					foreach ($templist as $name => $object)
					{
						$theusermask = $object->username .'@'. $object->host;
						
						
						/* found a match! */
						if (preg_match('/^'.$gline->regex.'$/', $theusermask))
						{
						
							/* if it matches an operator, ignore and tatle */
							if (($object->modes & UMODE_OPERATOR) > 0)
							{
								$irc->send(sprintf(OPERWALL, self::$client->nick, 'AKill oper override ['.$object->nick.'] matches: '.$mask));
								continue;
							}
						
							$total_ban++;
							
							$irc->send(sprintf(OPERWALL, self::$client->nick, 
								"AKill active for {$name} - matches: ".$gline->mask." (".$gline->reason.")"));
								
							self::$client->notice($name, '*** Banned: '.$gline->reason);
							
							$istemp = (($expiretime) ? 'Temporary' : 'Permanent');
							 
							$irc->send(sprintf(KILL, 
								$me->name, 
								$name, 
								$me->name, 
								self::$client->host, 
								self::$client->username,
								$me->name,
								$istemp." K-Line (".$gline->reason.")"));
								
								user::remuser($name);
						}
					}
					
					/*tell the issuer */
					self::$client->notice($nick, 
						'Added AKill for '.$gline->mask.' ('.$gline->reason.') [Expires: ' . (($expiretime) ? $numeric.$duration : 'Never').'] Affected users: '. $total_ban);
							
					/* tell the opers */
					$irc->send(sprintf(OPERWALL, self::$client->nick,
						$nick.' Added AKill for '.$gline->mask.' ('.$gline->reason.') [Expires: ' . (($expiretime) ? $numeric.$duration : 'Never').'] Affected users: '. $total_ban));
						
					/* now add the kline to the servers */
	
					$irc->send(sprintf(KLINE, self::$client->nick, $time, $userhost[0], $userhost[1], $gline->reason));
					return;
				} // end success
				
				self::$client->notice($nick, 'There was a problem adding your AKill. Please contact a coder.');
				return;
					//$irc->send(sprintf(NOTICE, $me
			} /* end valid pregmatch */
			
			self::$client->notice($nick, 'SYNTAX: AKILL ADD|DEL|LIST <+3(m|d|h|w)> <mask> <reason>');
			return;
			
		} //end add
		
		else 
			return self::$client->notice($nick, 'SYNTAX: AKILL ADD|DEL|LIST <+3(m|d|h|w)> <mask> <reason>');
		
	} // end gline
	
	/**
	 * Function to send 'access denied' string
	 *
	 * @param string $nick
	 */
	static function denied($nick, $command="unknown")
	{
		global $irc, $me;
		
		$user = user::getuser($nick);
		
		self::$client->notice($nick, 'Access denied. You have been logged.');
		$irc->send(sprintf(OPERWALL, self::$client->nick, "{$nick} (". $user->host .") access denied to: \2".$command."\2"));
	}
}