<?php

/**
 * BuddyServ IRC Services
 * 
 * Main program daemon
 * 
 * @name Vhost
 * @author Raymond Lynk <me@raymondj.ca>
 * @copyright (C) 2010, Raymond Lynk
 * @link http://buddyserv.raymondj.ca
 * @package Core
 * @version $Id: Class.vhost.php 331 2009-11-22 04:21:18Z lynk $
 */

/*
 * 		 ______           __     __         _______                        
 * 		|   __ \.--.--.--|  |.--|  |.--.--.|     __|.-----.----.--.--.     
 * 		|   __ <|  |  |  _  ||  _  ||  |  ||__     ||  -__|   _|  |  |     
 * 		|______/|_____|_____||_____||___  ||_______||_____|__|  \___/      
 *  				    			   |_|                                 
 *                                                                         
 */
 
class vhost
{
	protected $id;
	protected $owner;
	protected $host;
	protected $setby;
	protected $added;
	private $_uptodate=false;
	
	/**
	 * Magic getter function. Returns null
	 * if the variable doesnt exist.
	 * 
	 * @param string $what
	 * @return mixed
	 */
	public function __get($what)
	{
		if (isset($this->{$what}))
			return $this->{$what};
			
		return null;
	}
	
	/**
	 * Magic setter
	 * 
	 * @param string $what
	 * @param mixed $value
	 * @return unknown_type
	 */
	public function __set($what, $value)
	{
		if (($what == 'id') or ($what == '_uptodate'))
			return false;
			
		$this->{$what} = $value;
		$this->_uptodate = false;
	}
	
	/**
	 * Find a virtual host by either its own
	 * id, or the id of its owner.
	 * @param $id
	 * @param [id|userid] $typeId
	 * @return bool
	 */
	public function findById($id, $typeId='id')
	{
		global $db;
		
		if (!intval($id))
			return false;
			
		switch ($typeId)
		{
			case 'id':
				$type = 'id';
				break;
			
			case 'userid':
				$type = 'userid';
				break;
			
			default:
				$type = 'userid';
				break;
		}
			
		$query = $db->query(sprintf("SELECT * FROM `buddyserv_vhosts` WHERE `%s` = %d LIMIT 1", $type, $id));
		
		if ($db->num_rows($query) == 0)
			return false;
			
		$array = $db->fetch_array($query);
		
		$this->id = $array['id'];
		$this->owner = $array['userid'];
		$this->host = $array['vhost'];
		$this->setby = $array['staffmember'];
		$this->added = $array['added'];
		$this->_uptodate = true;
		
		return true;
	}
	
	/**
	 * Looks for a virtual host
	 * using a username as its search
	 * method.
	 * 
	 * @param string $username
	 * @return bool unknown_type
	 */
	public function findByUser($username)
	{
		global $db;
		
		$username = mysql_real_escape_string($username);
		
		/* We need to get the ID of the user from the database
		 * in a JOIN in order to know which records to return
		 * from the virtual host listing.
		 */
		$query = $db->query(sprintf("SELECT vhost.* FROM `buddyserv_vhosts` vhost JOIN `users` users ON vhost.userid=users.id WHERE users.username = '%s' LIMIT 1", $username));
		
		if ($db->num_rows($query) == 0)
			return false;
			
		$array = $db->fetch_array($query);
		
		$this->id		= $array['id'];
		$this->owner 	= $array['userid'];
		$this->host 	= $array['vhost'];
		$this->added 	= $array['added'];
		$this->setby	= $array['staffmember'];
		$this->_uptodate = true;

		return true;
	}
	
	/**
	 * Delete the current virtual host
	 * from the database.
	 * 
	 * @return bool
	 */
	public function delete()
	{
		global $db;
		
		/* valid object? */
		if (!$this->id)
			return false;
			
		$query = $db->query(sprintf("DELETE FROM `buddyserv_vhosts` WHERE `id` = %d LIMIT 1", $this->id));
		
		/* check for failure */
		if ($db->num_affected($query) == 0)
			return false;
			
		/* must have succeeded */
		return true;
	}
	
	/**
	 * Delete a virtual host using
	 * the vhosts owner as the search
	 * criteria.
	 * 
	 * @return bool
	 */
	public function deleteByUser()
	{
		global $db;
		
		/* check for valid ownerid */
		if (intval($this->owner) > 0)
		{
			/* valid */
			$query = $db->query(sprintf("DELETE FROM `buddyserv_vhosts` WHERE `userid` = %d LIMIT 1", $this->owner));
			
			/* check for failed delete */
			if ($db->num_affected($query) == 0)
				return false;
				
			return true;
		}
		
		/* failed owner sanity checks */
		return false;
	}
	
	/**
	 * Saves the current virtual host
	 * to the database as a new entry, or
	 * updating the previous entry if exists.
	 * 
	 * @return bool
	 */
	public function save()
	{
		global $db;
		
		/* If the object says we are still up to date, we aren't going
		 * to save. 
		 */
		if ($this->_uptodate)
			return false;
			
		/* sanitize some of the variables */
		if (!(intval($this->owner) or ($this->host) or ($this->setby)))
			return false; // missing values
		
		$this->added = time();
		
		/* If already an ID, we must be updating. */
		if ($this->id > 0)
		{		
			$query = $db->query(sprintf("UPDATE `buddyserv_vhosts` SET `vhost` = '%s', `staffmember` = '%s', `added` = %d  WHERE `userid` = %d LIMIT 1",
				mysql_real_escape_string($this->host),
				mysql_real_escape_string($this->setby),
				$this->added,
				intval($this->owner)));
				
			/* check for affected */
			if ($db->num_affected($query) > 0)
			{
				/* We updated the record. */
				return true;
			}
			
			/* We failed to modify any records. Drop back after 
			 * resetting the current vhosts object id, (This can
			 * occur if a vhost is deleted through tbdev)
			 * to the insert phase. DONT return; from this or shit.
			 */	
			$this->id = null;
			
		}
		
		/* Create a new vhost entry. */
		$query = $db->query(sprintf("INSERT INTO `buddyserv_vhosts` (`userid`, `vhost`, `added`, `staffmember`) VALUES (%d, '%s', %d, '%s')",
			intval($this->owner),
			mysql_real_escape_string($this->host),
			$this->added,
			mysql_real_escape_string($this->setby)));
		
		$this->id = $db->insert_id($query);
		
		/* ensure (not)valid insert */
		if ($this->id == 0)
			return false;
			
		$this->_uptodate = true;
		return true;
	}
	
	
}

?>